%% Signal processing
function [indb, Fs, N, fsamp, mfsamp, octmfsamp, rmfsamp, romfsamp, smfsamp, mid] = SigProc(nsample, t , t_num, t_len, welchlap, welchtype, freq)
    %%
    %(DAFIR sensors samples at 18 kHz and is decimated to 9 kHz, when the data
    %is converted to .wav file.)
   
    switch nsample
        case 1
        %sample 1
        [Fs, N, fsamp, mfsamp, octmfsamp] = WavProc('Sample1.wav', welchlap, welchtype, t);
        Lm = 420/t;
        case 2
        %sample 2
        [Fs, N, fsamp, mfsamp, octmfsamp] = WavProc('Sample2.wav', welchlap, welchtype, t);
        Lm = 392/t;
        case 3
        %sample 3
        [Fs, N, fsamp, mfsamp, octmfsamp] = WavProc('Sample3.wav', welchlap, welchtype, t);
        Lm = round(296/t);
    end
    
    %length of signal
    bl = Fs*t;                  % Length of signal in samples
    Fmax = Fs/2;                % Max frequency
    dt = 1/Fs;                  % Time resolution
    
    df = Fs/N;                  % Frequency resolution
    T = 1/df;                   % Periode of signal
    f = floor(0:df:Fs/2-df);
        
    %sum of frequencies
    smfsamp = sum(mfsamp,1);
    
    %finding the closest point of approach and the 50 time blocks indexes around
    %this point
    L = length(smfsamp);
    id = round(Lm - L/50):round(Lm + L/50);
    [mvl, mid] = max(smfsamp(id));
    mid = round(Lm - L/50) + mid -1;
    indmax = mid + t_len*t_num/2;
    indmin = mid - t_len*t_num/2;
    indb = indmin:t_len:indmax;
    
    %find the index of the FREQ frequencies
    indf = zeros(1,length(freq));
    for i = 1:length(freq)
        fsize = size(find(f == freq(i)));
        if 0 == fsize(2)
            indf(i) = find(f == freq(i)-1);
        else
            indf(i) = find(f == freq(i));
        end
    end
    
    %finds the amplitude value 
    rmfsamp = mfsamp(indf,indb);
    romfsamp = mfsamp(indf,indb);
    
    %The octavbands and center frequencies
    fomax = max(f);
    order = floor(10*log10(fomax));
    n = 1:order;
    octfreq = zeros(3,length(n));
    for i = 1:length(n)
        octfreq(2,i) = 10^(n(i)/10);
        octfreq(1,i) = octfreq(2,i)*2^(1/6);
        octfreq(3,i) = octfreq(2,i)*2^(-1/6);
    end
    
    %find the index of the octavfreq
    indof = zeros(1,length(freq));
    for i = 1:length(freq)
        for j = 1:length(n)
           if octfreq(3,j) <= freq(i) <= octfreq(1,j)
               indof(i) = i;
           end
        end
    end
    
    %finds the amplitude value in octavfreq averaging
    romfsamp = octmfsamp(indof,indb);
end