%% Process of the .wav file
function [Fs, N, fsamp, mfsamp,octmfsamp] = WavProc(filename, welchlap, welchtype, t)
%Manage the audioread function to manage memory
    [sample, Fs] = audioread(filename);
    L = length(sample);
    if (welchlap(1) == 1)
        overlap = welchlap(3);
        N = Fs;
        df = Fs/N;
        f = floor(0:df:Fs/2-df);
        fomax = max(f);
        order = floor(10*log10(fomax));

        bl = Fs*t;
        K = welchlap(2);
        M = round(2*bl/(K+1));
        han = hanning(M);
        
        if (welchtype == 'h')
            %Home made welch estimation
            G = sum(han)/M;

            f = zeros(M,K);
            fm = zeros(N,K);
            samp = zeros(bl,floor((L/bl)));
            fsamp = zeros(N,floor((L/bl)*(100/overlap)));
            mfsamp = zeros(N/2,floor((L/bl)));
            octmfsamp = zeros(order,floor(L/bl));


            for i = 1:bl:(L - bl)
                samp(:,(floor(i/bl)+1)) = sample(i:(i+bl-1));
                for j = 1:M*(overlap/100):M
                    %separate into the overlaping segements
                    f(:,round(2*(j/M) +1)) = samp((j:(M+j-1)),(floor(i/bl)+1)).*han;
                    %Take fourier transforms and PSD of this segements
                    fm(:,round(2*(j/M) +1)) = (2/(Fs*N*G))*abs(fft(f(:,round(2*(j/M) +1)),N)).^2;
                    %Sum them up for averaging
                    mfsamp(:,(floor(i/bl)+1)) = ((1/(K)))*fm(1:N/2,round(2*(j/M) +1)) + mfsamp(:,(floor(i/bl)+1));
                end
                mfsamp(:,(floor(i/bl)+1)) = sqrt((Fs/2)*mfsamp(:,(floor(i/bl)+1)));
            end
        elseif (welchtype == 'm')
            %Matlab welch estimation
            lest = 0;
            Mn = 1;
            teller = 1;
            while (lest == 0)
                if ((2^teller <= M) && (M < 2^(teller+1)))
                    Mn = 2^teller +1;
                    lest = 1;
                end
                teller = teller + 1;
            end
            samp = zeros(bl,floor((L/bl)));
            fsamp = zeros(N,floor((L/bl)*(100/overlap)));
            mfsamp = zeros(Mn,floor((L/bl)));
            octmfsamp = zeros(order,floor(L/bl));
            
            for i = 1:bl:(L - bl)
                %Seperat into t_len segements
                samp(:,(floor(i/bl)+1)) = sample(i:(i+bl-1));
                %Matlab welch estimation
                mfsamp(:,(floor(i/bl)+1)) = pwelch(samp(:,(floor(i/bl)+1)),han,M*(overlap/100));
                mfsamp(:,(floor(i/bl)+1)) = sqrt((Fs/2)*mfsamp(:,(floor(i/bl)+1)));
            end
        end
        
        
        
    else
        N = Fs;
        df = Fs/N;
        f = floor(0:df:Fs/2-df);
        fomax = max(f);
        order = floor(10*log10(fomax));
         bl = Fs*t;

        samp = zeros(bl,floor(L/bl));
        %samp_avg = zeros(L_avg,floor(L/bl));
        fsamp = zeros(N,floor(L/bl));
        mfsamp = zeros(N/2,floor(L/bl));
        octmfsamp = zeros(order,floor(L/bl));
        
        for i = 1:bl:(L - bl)
            %separate the audiosample into Fs*t time blocks
            samp(:,(floor(i/bl)+1)) = sample(i:(i+bl-1));
            %fft and shift it
            fsamp(:,(floor(i/bl)+1)) = (1/(N))*fft(samp(:,(floor(i/bl)+1)),N);
            %magnitude of the frequencies
            mfsamp(:,(floor(i/bl)+1)) = abs(fsamp(1:N/2,(floor(i/bl)+1)));
            %averaging octav averiging
            octmfsamp(:,(floor(i/bl)+1))= octavg(mfsamp(:,(floor(i/bl)+1)), f);
        end  
    end
end

%%Used for plotting the unprossessed time signal
%     [sample, Fs] = audioread('Sample1.wav');
%     L = length(sample);
%     
% %     %For sample 1
% %     redu = 1.6;
% %     sample(L-redu*Fs*60:L) = [];
% %     sample(1:redu*Fs*60) = [];
% %     time = redu:1/(60*Fs):L/(60*Fs)-1/(60*Fs)-redu;
% %     time(length(time)) = [];
%     
%     %For sample 2 and 3
%     time = 0/60:1/(60*Fs):L/(60*Fs)-1/(60*Fs);
%     figure
%     plot(time,sample,'DisplayName','Sum of frequency amplitude at each time block')
%     xlabel('Time (min)')
%     ylabel('Time signal')
%     xlim([time(1) time(length(time))])

%% Fourier transformation
% df = Fs/N;
% fn = 1:df:Fs;
% foursamp = zeros(length(fn),1);
% for i = 1:length(fn)
%     k = fn(i)/df;
%     for j = 1:bl
%         foursamp(i) = foursamp(i) + (1/N)*samp(j,140)*exp(-1i*2*pi*(j-1)*k/N);
%     end
% end