function [xbest,fbest, funccalls,track] = ASHS(ASHSparams,Pparams,RL,Sparam)
% HARMONY basic Harmony Search minimization for continuous variables

% Unpack the parameter vector
track = ASHSparams.track;
HMS    = ASHSparams.HMS;
HMCR   = ASHSparams.HMCR;
PAR    = ASHSparams.PAR;
b      = ASHSparams.b;
rho    = ASHSparams.nglobal;
DSinc  = ASHSparams.Lincfact;




xL       = Pparams.lb;
xU       = Pparams.ub;
func     = Pparams.func; 
stopCrit = Pparams.stopCrit;  
maxCalls = Pparams.funcCallsMax;

ASHSparams.conv=inf;
ASHSparams.d   = 0.00;

% Dimension arrays

N        = length(xL);      % Number of decision variables
HM       = zeros(HMS,N);
F        = zeros(HMS,1);
xnew     = zeros(1,N);
xas      = [];              % Matrix to contain harmonies returned from AS
t        = [];              % Matrix with index of the harmonies that are sent to DHS
fbest    = inf;             % Makes sure crit for first iteration is met
rhofrac  = 1;               % For addaptive number of improvisations before DHS. 


% Randomly initialize HM, taking care to keep each variable within bounds.
% Evaluate the corresponding objective function values.

for j = 1:HMS
    HM(j,:) = xL + (xU-xL).*rand(1,N);
    [F(j), track]    = func(RL,HM(j,:),Sparam,track);    
end
funccalls=HMS;

%---------------------------------------------------------------

while (fbest>stopCrit) & (funccalls<maxCalls)
  
%__________________________________________________________    
%                         HS - part



   for j=1:rho*rhofrac
    % Improvise a new harmony: loop though each variable
    for i = 1:N
        % Randomly perform one of the three HS operations
        if rand < HMCR
            % Memory considering: randomly select a note stored in HM
            xnew(i) = HM(ceil(rand*HMS),i);
            if rand < PAR
                % Pitch adjusting: randomly adjust the pitch slightly
                % within +/- b(i), and ensure bounds are satisfied
                xnew(i) = xnew(i) + (2*rand-1)*b(i);
                xnew(i) = min(max(xnew(i),xL(i)),xU(i));
            end
        else
            % Random playing: randomly select any pitch within bounds
            xnew(i) = xL(i) + rand*(xU(i)-xL(i));
        end
    end % Finished improvising a new harmony
   
    % HM update: check whether the new harmony is better than the worst
    % harmony currently in HM
    [fnew, track] = func(RL,xnew,Sparam,track);
    funccalls=funccalls+1;
    [fworst,idxworst] = max(F);
    if fnew < fworst
        HM(idxworst,:) = xnew;
        F(idxworst)    = fnew;
    end

    
    
    
    %Find the best harmonies for writing to screen and forwarding to DHS.
    [fbest,idxbest] = min(F);
    Fcopy=F;
    Fcopy(idxbest)=inf;
    [F2best,idx2best] = min(Fcopy);
    
    
  
    %---------------------------------------------------------------
    
   % disp(['Function Calls: ' num2str(funccalls) '  Error: ' num2str(fbest)])
   end
   
   %__________________________________________________________________    
%                         DS-part
  %  disp(['Best imp: ',num2str(fbest)])
  %  disp('Start DS')

        %Make the DHS adaptive, ie. respond to current fbest.
        t(1)=idxbest;
        Fcopy=F;

        %Perform DHS on length(t) harmonies. 
        tmp=inf;
        fas=ones(size(t))*inf;
        j=1;
        while (j<length(t)+1) & (tmp>stopCrit)
            ASHSparams.sp=HM(t(j),:);
            size(xas);
            [ xas(j,:), fas(j), icount, numres,track ] = nelmin (ASHSparams, Pparams, RL, Sparam,track);
            funccalls=funccalls+icount; 
            tmp=fas(j);
            j=j+1;
        end
        ASHSparams.nlocal=ASHSparams.nlocal+ASHSparams.nlocal*DSinc;

        %Rank the simplexes
        fascopy=fas;
        fasbest=zeros(size(fas));
        idxasbest=zeros(size(fas));
        for i=1:length(fas)
        [dummy idxasbest(i)]=min(fascopy);
        fascopy(idxasbest(i))=inf;
        end
        fasbest=sort(fas);

    % disp(['Best simplex: ', num2str(fasbest(1)) 'Function calls: ' num2str(funccalls)])

        

        %Find the length(fas) worst harmonies in HM

        Fcopy=F;
        fmax=zeros(size(fas));
        idxmax=zeros(size(fas));
        for i=1:length(fas)
            [fmax(i) idxmax(i)]=max(Fcopy);
            Fcopy(idxmax(i))=0;
        end 
        HMworst=HM(idxmax(:));
    if fasbest(1)<fbest
        fbest=fasbest(1);
    end
    
    %Replace harmonies in HM if worse than new 
    Fcopy=F;
    count=1;
    
    for i=1:min(1,length(fas))
        [dummy tmpind]=min([Fcopy(idxmax(end)) fasbest(1)]);
        if tmpind==1
            HMworst=HMworst(1:end-1);
            idxmax=idxmax(1:end-1);
        else
            HM(idxmax(end-count+1),:)=xas(idxasbest(i),:);
            F(idxmax(end-count+1))=fasbest(1);
            if length(fasbest)>1
            fasbest=fasbest(2:end);
            end
            count=count+1;
        end
    end
               

%________
    


end % Stop criteria

% Return the best harmony found
[fbest,idxbest] = min(F);
xbest = HM(idxbest,:);
% funccalls

% toc