%% Ambient noise
% Function to return the ambient noise for the given frequencies

function [N] = AmbNoise(freq)
%The ambient noise without the shipping noise, a combination of turbulence
%Nt, wind driven waves Nw and thermal noise Nth. Ref 1 uPa
    w = 4;                          %Wind speed (m/s)
    
    Nt = (10^-6)*10.^((17 - 30*log10(freq))/10);        %Noise from turbulence
    Nw = (10^-6)*10.^((50 + 7.5*w^(1/2) + 20*log10(freq) - 40*log10(freq + 0.4))/10);   %Noise from wind
    Nth = (10^-6)*10.^((-15 + 20*log10(freq))/10);      %Therminal noise
    
    N = (Nt + Nw + Nth);

end

