%% Sets values in the RAM .dat file 
function [RL] = RAMV(Rparam, Sparam, Pparam )
%Saves alle the properties of the enviroment for the RAMsearch later
    cp_ref = Sparam.cp;             % Referance sound speed
    d = Sparam.d;                   % Reciever,source and seafloor depth (m) 
    t_len = Sparam.t_len;           % Length of time block in seconds
    t_bet = Sparam.t_bet;           % Number of timeblocks between each chosen timeblock
    t_num = Sparam.t_num;           % Number of time blocks needed for the comparison
    f = Sparam.freq;                % Frequencies (Hz)
    shippath = Sparam.shippath;     % Type of ship path
    dr = Sparam.dr;                 % Range resolution
    
    %referance pressure
    k = 2*pi*f/cp_ref(1);           % Wave number
    p_ref = exp(1i*k*1)/1;          
    
    
    SP = Rparam.SP;                 % Sound speed profiel speed (m/s) and depth (m)
    dep = Rparam.depR;              % Start of sea bottom (m), silt, clay and sand (calculated from the bottom)
    cp = Rparam.cpR;                % P wave speed (m/s), silt, clay and sand
    rho = Rparam.rhoR;              % Density (g/m^3), silt, clay and sand
    atn = Rparam.atnR;              % Attenuation (dB/alfa), silt, clay and sand   
    ub = Pparam.ub;                 % Upperlimits of algorithm search
    
    %the max range possible for the code
        if (shippath == 's')
            rb_max = t_len*t_bet*round(t_num/2)*ub(1);
        else
            rb_max = t_len*t_bet*round(t_num)*ub(1);
        end
    r_max = ceil(sqrt((rb_max).^2 + ub(2)^2));

    %% Set paramters
    text = importdata('magne.dat');
    % Reciever and source depth
    text(2,2)=d(2);
    text(2,3)=d(1);
    % Max range and range resolution
    text(3,1)=r_max;
    text(3,2)=dr;
    % Seafloor bathymetry
    text(6,2)=d(1);
    text(7,2)=d(1);
    text(7,1)=r_max;
    % Sound speed profile and ref sound speed
    text(5,1) = cp_ref(1);
    for i = 9:21
        text(i,1) = SP(2,i-8);
        text(i,2) = SP(1,i-8);
    end
    % Sound speed bottom
    for i = 23:24
        text(i,1) = dep(i-22);
        text(i,2) = cp(i-22);
    end
    % Density bottom
    for i = 26:27
        text(i,1) = dep(i-25);
        text(i,2) = rho(i-25);
    end
    % Attenuation bottom
    for i = 29:30
        text(i,1) = dep(i-28);
        text(i,2) = atn(i-28);
    end
    
        %% Create RAM field for each frequencies
    % referanse pressure
    % Max horizontal range and range resolution
    text(2,1) = f(1);
    save('magne.dat','text','-ascii')

    copyfile('magne.dat','fort.1')
    system('ramg2.exe');
    copyfile('fort.2','magne.tlr')
    A=load('magne.tlr');
    
    RL = zeros(length(f),length(A(:,2)));
    RL(1,:) = abs(p_ref(1))*10.^(-A(:,2)/20);

        % run RAM
        for i = 2:length(f)
            % run RAM
            text(2,1) = f(i);
            save('magne.dat','text','-ascii')

            copyfile('magne.dat','fort.1')
            system('ramg2.exe');
            copyfile('fort.2','magne.tlr')
            A=load('magne.tlr');
            RL(i,:) = abs(p_ref(i))*10.^(-A(:,2)/20);
        end
        
    
    
    save('magne.dat','text','-ascii') 
end

%% RAM magne.dat
% line 1: Title, leave zeros
% line 2: freq in Hz, source depth, receiver depth in meters
% line 3: max range, range step, output step (range)
% line 4: max depth comp., depth step, vertical extent of domain to output
% line 5: ref sound speed, number of pade terms, stability constraints,
%           radius of stability constraint
% line 6: start: range, bathymetry
% line 7: end: range, bathymetry
% line 8: show end of bathymetry
% line 9: start: depth, sound speed
% line 21: end: depth, sound speed
% line 22: shows end of sound speed profile
% line 23: start: depth from bottom, sound speed first layer
% line 24: depth from bottom, sound speed second layer
% line 25: end: depth from bottom, sound speed third layer
% line 26: shows end of sound speed bottom
% line 27: start: depth from bottom, density first layer
% line 28: depth from bottom, density second layer
% line 29: end: depth from bottom, density third layer
% line 30: shows end of density bottom
% line 31: start: depth from bottom, attenuation first layer
% line 32: depth from bottom, attenuation second layer
% line 33: depth from bottom, attenuation third layer
% line 34: false bottom
% line 35: end: false bottom
% line 36: shows end of attenuation bottom