function [RL] = RAMsearch(par, Pparam )
%% Extract the range values from the Ram created field, only made for unknown v_s and d_cpa
    
    %Parameters
    t_len = Pparam.t_len;           % Length of time blocks
    t_bet = Pparam.t_bet;           % Time block spacing 
    t_num = Pparam.t_num;           % Number of time blocks
    
    v_s  = par(1);                  % Ship velocity
    d_cpa = par(2);                 % Distance at closest point of approach
    RV = Pparam.RV;                 % Matrix of RAM propagation
    dr = Pparam.dr;                 % Range resolution
    shippath = Pparam.shippath;     % Type of ship path
    sbal = Pparam.sbal;             % How many of the t_num time blocks is before or after the cpa when using shippath 
    tstart = Pparam.tstart;         % Decides if the ship path is created from CPA, value 0, or from a point 'x' timeblocks away, where positive values are in after CPA direction
    
    %time between each timeblock measured and with v_s can find distence
    %between each timeblock and can creating the time series of the signal in range 
   if (v_s == 0)                        % Standing still
        rb = zeros(t_num+1,1);
   else
        if (shippath == 's')            % Ship path symetric around CPA
            rb_min = -t_len*t_bet*round(t_num/2)*v_s + tstart*v_s;
            rb_max = t_len*t_bet*round(t_num/2)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'b')        % Ship path before CPA
            rb_min = -t_len*t_bet*round(t_num)*v_s + tstart*v_s;
            rb_max = tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'a')        % Ship path after CPA
            rb_min = tstart*v_s;
            rb_max = t_len*t_bet*round(t_num)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'l')        % Ship path abit before CPA and a little after
            rb_min = -t_len*t_bet*round(t_num-sbal)*v_s + tstart*v_s;
            rb_max = t_len*t_bet*round(sbal)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        elseif (shippath == 'k')        % Ship path abit after CPA and a little before
            rb_min = -t_len*t_bet*round(sbal)*v_s + tstart*v_s;
            rb_max = t_len*t_bet*round(t_num - sbal)*v_s + tstart*v_s;
            steps = (rb_max-rb_min)/t_num;
            rb = rb_min:steps:rb_max;
        end
   end
   %create ship path as distance from the reciever
    rt = sqrt((rb).^2 + d_cpa^2);
    
    % The chosen RAM values
    RL = RV(:,round(rt*(1/dr)));

end

%% RAM magne.dat
% line 1: Title, leave zeros
% line 2: freq in Hz, source depth, receiver depth in meters
% line 3: max range, range step, output step (range)
