function [vmfsamp, df] = SigProc(Sparam)
    %% Chose data sample and extract disered values
    %(DAFIR sensors samples at 18 kHz, decimated to 9 kHz)
    nsample = Sparam.nsample;       % Which time sample are used
    freq = Sparam.freq;             % Frequency values
    t_num = Sparam.t_num;           % Number of time blocks
    t_bet = Sparam.t_bet;           % Time block spacing 
    t_len = Sparam.t_len;           % Length of time blocks
    ftype = Sparam.ftype;           % Type of processing recorded data
    octavband = Sparam.octavband;   % Type of octave processing
    shippath = Sparam.shippath;     % Type of ship path
    sbal = Sparam.sbal;             % How many of the t_num time blocks is before or after the cpa when using shippath
    tstart = Sparam.tstart;         % How many of the t_num time blocks is before or after the cpa when using shippath
    
    switch nsample
        case 1
        %sample 1
        Lm = 420/t_len;
        [Fs, N, mfsamp] = WavProc('Sample1.wav', Sparam);
        lenround = 50;
        case 2
        %sample 2
        Lm = 392/t_len;
        [Fs, N, mfsamp] = WavProc('Sample2.wav', Sparam);
        lenround = 50;
        case 3
        %sample 3
        Lm = round(296/t_len);
        [Fs, N, mfsamp] = WavProc('Sample3.wav', Sparam);
        lenround = 50;
    end
    
    %length of signal
    df = Fs/N;
    f = floor(0:df:Fs/2-df);
    
    %% Selecting data
    %finding the closest point of approach
    sf = size(mfsamp);
    L = sf(2);
    id = round(Lm - L/lenround):round(Lm + L/lenround);
    [mvl, mid] = max(sum(mfsamp(:,id),1)/Fs);
    mid = round(Lm - L/lenround) + mid - 1 + tstart;
    %t_num points symetrical around, before or after closest point of
    %approach, or a bit before or a bit after
        if (shippath == 's')
            indmax = mid + t_bet*t_num/2;
            indmin = mid - t_bet*t_num/2;
            indb = indmin:t_bet:indmax;
        elseif (shippath == 'b')
            indmax = mid;
            indmin = mid - t_bet*t_num;
            indb = indmin:t_bet:indmax;
        elseif (shippath == 'a')
            indmax = mid + t_bet*t_num;
            indmin = mid;
            indb = indmin:t_bet:indmax;
        elseif (shippath == 'l')
            indmax = mid + t_bet*sbal;
            indmin = mid - t_bet*(t_num - sbal);
            indb = indmin:t_bet:indmax;
        elseif (shippath == 'k')
            indmax = mid + t_bet*(t_num - sbal);
            indmin = mid - t_bet*sbal;
            indb = indmin:t_bet:indmax;
        end
        
    %% makes sure the clippings aren't selected for sample 1
    if (nsample == 1) 
        inwr = 236:248;
        for i = 1:length(indb)
            switch indb(i)
                case 355
                indb(i) = 354;
                case 356
                indb(i) = 357;
                case 372
                indb(i) = 371;
                case 373
                indb(i) = 374;
                case 407
                indb(i) = 406;
                case 408
                indb(i) = 406;
                case 409
                indb(i) = 410;
                case 445
                indb(i) = 444;
                case 446
                indb(i) = 447;
            end
            for j = 1:length(inwr)
                if (indb(i) == inwr(j))
                    indb(i) = 235;
                end
            end
        end
    end
    
    %% returns either the inverse processed PSD or welch (welch found in WaveProc.m) or octavband or 1 linearized octavband
    if (ftype == 'n')
        %find the index of the FREQ frequencies
        indf = zeros(1,length(freq));
        for i = 1:length(freq)
            fsize = size(find(f == freq(i)));
            if 0 == fsize(2)
                indf(i) = find(f == freq(i)-1);
            else
                indf(i) = find(f == freq(i));
            end
        end

        %the amplitude values of 1 Hz band
        vmfsamp = mfsamp(indf,indb);
    elseif (ftype == 'w')
        %find the index of the FREQ frequencies
        indf = zeros(1,length(freq));
        for i = 1:length(freq)
            fsize = size(find(f == freq(i)));
            if 0 == fsize(2)
                indf(i) = find(f == freq(i)-1);
            else
                indf(i) = find(f == freq(i));
            end
        end

        %the amplitude values of 1 Hz band
        vmfsamp = mfsamp(indf,indb);
        
    elseif (ftype == 'o')
        if (octavband == 'n')
            %return octav band averaged
            %The octoftavbands and center frequencies
            fomax = max(f);
            order = floor(10*log10(fomax));
            n = 1:order;
            octfreq = zeros(3,length(n));
            for i = 1:length(n)
                octfreq(2,i) = 10^(n(i)/10);
                octfreq(1,i) = octfreq(2,i)*2^(1/6);
                octfreq(3,i) = octfreq(2,i)*2^(-1/6);
            end

            %find the index of the octavfreq
            indof = zeros(1,length(freq));
            for i = 1:length(freq)
                for j = 1:length(n)
                   if (octfreq(3,j) <= freq(i) && freq(i) <= octfreq(1,j))
                       indof(i) = j;
                   end
                end
            end

            %octavband
            octmfsamp = zeros(order,length(indb));
            for i = 1:length(indb)
                %averaging octav averiging
                octmfsamp(:,i)= octavg(mfsamp(:,indb(i)), f);
            end

            %finds the amplitude values of octavband 
            vmfsamp = octmfsamp(indof,:);
        elseif (octavband == 'l')
            %return octav band which are linearized
            %The octavbands and center frequencies
            fomax = max(freq);
            order = floor(10*log10(fomax))+2;
            n = 1:order;
            octfreq = zeros(3,length(n));
            for i = 1:length(n)
                octfreq(2,i) = 10^(n(i)/10);
                octfreq(1,i) = octfreq(2,i)*2^(1/6);
                octfreq(3,i) = octfreq(2,i)*2^(-1/6);
            end
            f = 1:max(max(octfreq));

            %find the index of the octavfreq
            indof = zeros(1,length(freq));
            for i = 1:length(freq)
                fsize = size(find(f == freq(i)));
                if 0 == fsize(2)
                    indof(i) = find(f == freq(i)-1);
                else
                    indof(i) = find(f == freq(i));
                end
            end

            %octavband
            octmfsamp = zeros(order,length(indb));
            octfreq = round(octfreq);
            octmffreq = zeros(octfreq(1,size(octfreq,2)-1),1);
            octmfsig = zeros(octfreq(1,size(octfreq,2)-1),length(indb));
            for i = 1:length(indb)
                %averaging octav averiging
                octmfsamp(:,i)= octavg(mfsamp(:,indb(i)), f);

                %turn octav averiging from 1/3 octav band to 1 Hz band, using the averaged
                %as the start point and the next averaged as the end point in each averaged
                %band
                for j = 1:(size(octfreq,2)-1)
                    xmin = octfreq(3,j);
                    xmax = octfreq(1,j);
                    ymin = octmfsamp(j,i);
                    ymax = octmfsamp(j+1,i);

                    octmffreq(xmin:xmax) = xmin:xmax;
                    octmfsig(xmin:xmax,i) = ymin + ((ymax-ymin)/(xmax-xmin))*((xmin:xmax)-xmin);
                end
            end

            %finds the amplitude values of octavband 
            vmfsamp = octmfsig(indof,:);
        end

    end
end