%% Simple 2D search
% A script that creats a 2D matrix of the cost function based on the two parameters
% and finds the minimum cost value in this matrix

clear all

func=@objectivefunction;                % Call on cost function

vs=[2*0.5144,35*0.5144];                % Ship velocity      % sample 3 [2*0.5144, 10*0.5144], sample 1 [10*0.5144, 20*0.5144]
dcpa=[10,2000];                         % Distance at closest point of approach
%d_s =[1.8,4.2];                         % Source depth
%d_b =[170,230];                         % Seafloor and reciever depth
%cp=[1200,1800];                         % Sound speed in water
%cs=[1200,1800];                         % Sound speed in sediment
%rho=[0.7,1.3]*1000;                     % Density in water
%rho_s=[1.2,1.8]*1000;                   % Density in sediment

%Set problem upper and lower bounds.
Pparams.ub=[vs(2) dcpa(2)];
Pparams.lb=[vs(1) dcpa(1)];
ub=Pparams.ub;
lb=Pparams.lb;

par1 = lb(1):0.1:ub(1);
par2 = lb(2):1:ub(2);
t_num = 24;

%% Produce Synthetic Data
ds = 50;
%Shared
Sparam.t_len = 1;               % Length of time block in seconds
Sparam.t_bet = 3;               % Number of timeblocks between each chosen timeblock
Sparam.t_num = t_num;           % Number of time blocks needed for the comparison
Sparam.freq = [103 308 476 550 720 890];   %Frequencies (Hz), sample 3 [103 308 476 550 720 890], sample 1 [138 235 343 479 680 878], [138 232 352 495 680 810]
Sparam.d=[200, 3.325, 200];     % Reciver, source and seafloor depth (m), sample 3 [3.325], sample 1 [1.4]
Sparam.dr=1;                    % Range resolution
Sparam.df = 1;                  % Frequency resolution
%LloydMirr
nLloyd = 5;                     % Order of syntetic Lloyd mirror
Sparam.mLloyd = 3;              % Order of modeled Lloyd mirror
Sparam.cp=[1482, 1502];         % P wave speed (m/s)
Sparam.rho=[1.0, 1.500]*1000;     % Density (kg/m^3)
Sparam.vs=[10*0.5144];          % Ship speed, syntetic data (m/s)
Sparam.dcpa=[250];              % Closest point of approach, syntetic data (m)
%RAM
Rparam.SP=...                   % Sound speed profiel speed (m/s) and depth (m)
[1483, 1492, 1486, 1482, 1478, 1471, 1471, 1475, 1480, 1482, 1482.5, 1483, 1483;...
0 5 10 15 20 30 40 50 60 70 80 90 200];
Rparam.depR = [0 100];          % Start of sea bottom (m), clay and mud (calculated from the bottom)
Rparam.cpR=[1500, 1615];        % P wave speed (m/s), clay and mud
Rparam.rhoR=[1.5, 1.8];         % Density (g/m^3), clay and mud
Rparam.atnR=[0.2, 0.9];         % Attenuation (dB/alfa), clay and mud
%Different processes
Sparam.shippath = 'l';          % Simulating of the ship path, 's' symetrical around cpa, 'b' before cpa and 'a' after cpa, 'l' before and abit after and 'k' which is ater and abit before
Sparam.sbal = 2;                % How many of the t_num time blocks is before or after the cpa when using shippath 'l' or 'k'
Sparam.tstart = 0;              % Decides if the ship path is created from CPA, value 0, or from a point 'x' timeblocks away, where positive values are in after CPA direction
Sparam.noi = 'u';               % 'u' no noise added, 'n' noise added
Sparam.dmodel = 'r';            % Chose type of data model uses, 's' syntetic or 'r' real
Sparam.model = 'L';             % Which model used, 'L' Lloyd 'R' RAM
Sparam.nsample = 3;             % Which time sample is uesed 1, 2 or 3
Sparam.ftype = 'w';             % Return the 'n' 1 Hz band values, 'w' welch spektral estimation and 'o' octavband of the real og syntetic data
Sparam.welchlap = [3 50];       % Number of overlapping segements and overlap in percent when using welch
Sparam.welchtype = 'm';         % 'h' for home made welch estimation, 'm' for matlab pwelch
Sparam.Ltype = 'n';             % Return the 'n' 1 Hz band values and 'o' octavband of the Lloyd mirror
Sparam.octavband = 'l';         % 'n' constant value of the octavband averaging or 'l' 1 Hz band linearisation of the octavband averaging

% RAM has to be stored in a matrix and values taken from this matrix or
% else the code will stop working
if (Sparam.model == 'R')
    Sparam.RV = RAMV(Rparam, Sparam, Pparams);
end
    
% Create synthetic data or use measured recordings
dmodel = Sparam.dmodel;
if (dmodel == 's')
    %Syntetic data
    [RL] = LMmodel(Sparam,nLloyd);
elseif (dmodel == 'r')
    %Real data
    [RL, Sparam.df] = SigProc(Sparam);
end

% Creates ambientnoise and adds it to the synthetic data if wanted
noi = Sparam.noi;
if (( dmodel == 's') && (noi == 'n'))
    f = Sparam.freq;
    SNRN = 5;                       % SNR parameter
    Nadj = ((mean(abs(RL(:,t_num/2 +1)))/(10^(SNRN/20)))/mean(AmbNoise(f))); % Calculated noise adjustment to scale the noise to the chosen SNR
    Sparam.Nadj = Nadj;             % Noise adjustment
    Sparam.Nvar = 0.1;              % Noise variance
    [RL] = LMmodelNoise(Sparam,nLloyd); % Add noise to the synthetic data
end
%% Creat matrix and find index of min error

error = zeros(length(par1),length(par2));
track = zeros(9,1);
% Call the cost function for each step in the parameter space and get the
% error and the possible tracking 
tic;
for i = 1:length(par1)
   for j = 1:length(par2)
       par = [par1(i) par2(j)];
       [err, track] = func(RL,par,Sparam,track);
       error(i,j) = 20*log10(err);
   end
end
toc

% Find index of minimum error
[minerror,ind] = min(error(:));
[m,n] = ind2sub(size(error),ind);

%% Plot cost value at each combination of the paramters
% Plot log
figure
erfig = pcolor(par2,par1,error),shading('interp');
h = colorbar('location','EastOutside');
ylabel(h, 'Error in dB')
%title('Simple search')
xlabel('Range (m)')
ylabel('Boat speed (m/s)')

% Values of the parameter
par = [par1(m) par2(n)]
pval = LMmodel2(par,Sparam,1);

% Inverse plot
erter = 10.^(error/20);
ertlog = 20*log10(1-erter);

% Plot inverse log
figure
ertlfig = pcolor(par2,par1,ertlog),shading('interp');
h = colorbar('location','EastOutside');
ylabel(h, 'Error in dB')
%title('Simple search')
xlabel('Range (m)')
ylabel('Boat speed (m/s)')

% Plot normal scale
figure
ertfig = pcolor(par2,par1,erter),shading('interp');
h = colorbar('location','EastOutside');
ylabel(h, 'Error in normal scale')
%title('Simple search')
xlabel('Range (m)')
ylabel('Boat speed (m/s)')

minerror = min(min(erter))

% saveas(erfig, sprintf('sample 3 log Lloyd 5'),'fig');
% saveas(ertlfig, sprintf('sample 3 ilog Lloyd 5'),'fig');
% saveas(ertfig, sprintf('sample 3 norm Lloyd 5'),'fig');
