%% Manage wav file
function [Fs, N, mfsamp] = WavProc(filename, Sparam)
%Manage the audioread function to manage memory
    t_len = Sparam.t_len;               % Length of time blocks
    ftype = Sparam.ftype;               % Type of process for recorded data
    welchlap = Sparam.welchlap;         % Values used in the welch spectral method
    welchtype = Sparam.welchtype;       % Which welch is used home med or MATLAB pwelch
    
    %Extract data from file
    [sample, Fs] = audioread(filename);
    L = length(sample);                 % Length of time signal in samples
    N = Fs;                             % Choose the number for fourier transform to be as large as sample to get 1 Hz band                  
    bl = Fs*t_len;                      % Length of timeblock in samples
    
    if (ftype == 'w')
        %welch spektral estimation
        K = welchlap(1);
        M = round(2*bl/(K+1));
        han = hanning(M);
        
        if (welchtype == 'h')
            %Home made welch estimation
            G = sum(han.^2)/M;

            f = zeros(M,K);
            fm = zeros(N,K);
            samp = zeros(bl,floor((L/bl)));
            mfsamp = zeros(N/2,floor((L/bl)));

            for i = 1:bl:(L - bl)
                samp(:,(floor(i/bl)+1)) = sample(i:(i+bl-1));
                for j = 1:M*(welchlap(2)/100):M
                    %separate into the overlaping segements
                    f(:,round(2*(j/M) +1)) = samp((j:(M+j-1)),(floor(i/bl)+1)).*han;
                    %Take fourier transforms and PSD of this segements
                    fm(:,round(2*(j/M) +1)) = (2/(Fs*N*G))*abs(fft(f(:,round(2*(j/M) +1)),N)).^2;
                    %Sum them up for averaging
                    mfsamp(:,(floor(i/bl)+1)) = ((1/(K)))*fm(1:N/2,round(2*(j/M) +1)) + mfsamp(:,(floor(i/bl)+1));
                    mfsamp(:,(floor(i/bl)+1)) = sqrt((Fs/2)*mfsamp(:,(floor(i/bl)+1)));
                end
            end
        elseif (welchtype == 'm')
            %Matlab welch estimation
            lest = 0;
            Mn = 1;
            teller = 1;
            %The pwelch function gives only 2^x frequencies where, 2^x is
            %the closest of the length of the window from bellow
            while (lest == 0)
                if ((2^teller <= M) && (M < 2^(teller+1)))
                    Mn = 2^teller +1;
                    lest = 1;
                end
                teller = teller + 1;
            end
            samp = zeros(bl,floor((L/bl)));
            mfsamp = zeros(Mn,floor((L/bl)));
            
            for i = 1:bl:(L - bl)
                %Seperat into t_len segements
                samp(:,(floor(i/bl)+1)) = sample(i:(i+bl-1));
                %Matlab welch estimation
                mfsamp(:,(floor(i/bl)+1)) = pwelch(samp(:,(floor(i/bl)+1)),han,M*(welchlap(2)/100));
                mfsamp(:,(floor(i/bl)+1)) = sqrt((Fs/2)*mfsamp(:,(floor(i/bl)+1)));
            end
        end
          
       
    else
        %Only use fourier transform to extract frequency values
        samp = zeros(bl,floor(L/bl));  
        fsamp = zeros(N,floor(L/bl));
        mfsamp = zeros(N/2,floor(L/bl));

        for i = 1:bl:(L - bl)
            %separate the audiosample into Fs*t time blocks
            samp(:,(floor(i/bl)+1)) = sample(i:(i+bl-1));
            %fft and shift it
            fsamp(:,(floor(i/bl)+1)) = (1/(N))*fft(samp(:,(floor(i/bl)+1)),N);  
            %magnitude of the frequencies
            mfsamp(:,(floor(i/bl)+1)) = abs(fsamp(1:N/2,(floor(i/bl)+1)));
        end  
   end
end