%% optimal_reflection
%  this code is to test ASHS and ASDE for range and ship velocity estimation 

%%

% Parameters passed to function:
% POPSIZE      = Size of population
% NGEN         = Number of generations before termination
% SELECTION    = Selection rate
% MUTRATE      = Mutation rate
% t_len        = Time block length (s)
% t_bet        = Time block spacing 
% t_num        = Number of time blocks
% F            = Frequency(constant) (Hz)

% VS           = Ship speed vector, (m/s)
%                [Speed-min,Speed-max]
% Dcpa         = Closest point of approach (m)
%                [min-length,max-length]
% D            = Reciver, Source Depth and seafloor depth (m)
%                [Reciver-min,Reciver-max,Source-min,Source-max]
% CP           = P-wave speed bounds vector, (m/s)
%                [min-water,max-water,min-sediment,max-sediment]
% RHO          = Density, (kg/m^3)
%                [min-water, max-water,min-sediment,max-sediment]




%Function runs Cost function and returns the following:
%PAR           = Matrix containing ranked population after final generation
%BEST          = Optimization error of the best chromosome


%%
clear all
clc


%% Problem spesific parameters

Pparams.func=@objectivefunction;        % Cost function
Pparams.stopCrit=0.0000000001;          % Stopping Criteria 1: Accepted cost function error
Pparams.funcCallsMax= 50000;            % Stopping Criteria 2: Maximum number of function calls

%Set boundaries for the parameters (for more than the vs and dcpa to be
%used in the search, the LMmodel2.m has to be changed to consider more
%unknown paramters)
vs=[2*0.5144, 10*0.5144];                % Ship velocity
dcpa=[10,200];                           % Distance at closest point of approach
d_s =[2.4,4.0];                          % Source depth
%d_b =[150,250];                         % Seafloor and reciever depth
%cp=[1000,2000];                         % Sound speed in water
%cs=[1000,2000];                         % Sound speed in sediment
%rho=[0.5,1.5]*1000;                     % Density in water
%rho_s=[1.0,2.0]*1000;                   % Density in sediment


%Set problem upper and lower bounds.
%Pparams.ub=[vs(2) dcpa(2) d_s(2) d_b(2) cp(2) cs(2) rho(2) rho_s(2)];
%Pparams.lb=[vs(1) dcpa(1) d_s(1) d_b(1) cp(1) cs(1) rho(1) rho_s(1)];
Pparams.ub=[vs(2) dcpa(2) d_s(2)];
Pparams.lb=[vs(1) dcpa(1) d_s(1)];
pN = length(Pparams.ub);
t_num = 100;                     % Number of time blocks needed for the comparison


%% Produce Synthetic Data
ds = 50;
%Shared
Sparam.t_len = 1;               % Length of time block in seconds
Sparam.t_bet = 1;               % Number of timeblocks between each chosen timeblock
Sparam.t_num = t_num;           % Number of time blocks needed for the comparison
Sparam.freq = [138 235 343 479 570 680 878 950];   %Frequencies (Hz)
Sparam.d=[200, 3.2, 200];       % Reciver, source and seafloor depth (m)
Sparam.df = 1;                  % Frequency resolution
Sparam.dr=1;                    % Range resolution
%LloydMirr
nLloyd = 3;                     % Order of syntetic Lloyd mirror
Sparam.mLloyd = 3;              % Order of modeled Lloyd mirror
Sparam.cp=[1482, 1502];         % P wave speed (m/s)
Sparam.rho=[1.0, 1.500]*1000;   % Density (kg/m^3)
Sparam.vs=[5.5*0.5144];         % Ship speed, syntetic data (m/s)
Sparam.dcpa=[65];               % Closest point of approach, syntetic data (m)
%RAM
Rparam.SP=...                   % Sound speed profiel speed (m/s) and depth (m)
[1483, 1492, 1486, 1482, 1478, 1471, 1471, 1475, 1480, 1482, 1482.5, 1483, 1483;...
0 5 10 15 20 30 40 50 60 70 80 90 200];
Rparam.depR = [0 100];          % Start of sea bottom (m), clay and mud (calculated from the bottom)
Rparam.cpR=[1500, 1615];        % P wave speed (m/s), clay and mud
Rparam.rhoR=[1.5, 1.8];         % Density (g/m^3), clay and mud
Rparam.atnR=[0.2, 0.9];         % Attenuation (dB/alfa), clay and mud
%Different processes
Sparam.shippath = 'l';          % Simulating of the ship path, 's' symetrical around cpa, 'b' before cpa and 'a' after cpa, 'l' before and abit after and 'k' which is ater and abit before
Sparam.sbal = 0;                % How many of the t_num time blocks is before or after the cpa when using shippath 'l' or 'k'
Sparam.tstart = 0;              % Decides if the ship path is created from CPA, value 0, or from a point 'x' timeblocks away, where positive values are in after CPA direction
Sparam.noi = 'u';               % 'u' no noise added, 'n' noise added
Sparam.dmodel = 's';            % Chose type of data model uses, 's' syntetic or 'r' real
Sparam.model = 'L';             % Which model used, 'L' Lloyd 'R' RAM
Sparam.nsample = 1;             % Which time sample is uesed 1, 2 or 3
Sparam.ftype = 'w';             % Return the 'n' 1 Hz band values, 'w' welch spektral estimation and 'o' octavband of the real og syntetic data
Sparam.welchlap = [3 50];       % Number of overlapping segements and overlap in percent when using welch
Sparam.welchtype = 'm';         % 'h' for home made welch estimation, 'm' for matlab pwelch
Sparam.Ltype = 'n';             % Return the 'n' 1 Hz band values and 'o' octavband of the Lloyd mirror
Sparam.octavband = 'l';         % 'n' constant value of the octavband averaging or 'l' 1 Hz band linearisation of the octavband averaging

% RAM has to be stored in a matrix and values taken from this matrix or
% else the code will stop working
if (Sparam.model == 'R')
    Sparam.RV = RAMV(Rparam, Sparam, Pparams);
end

% Create synthetic data or use measured recordings
dmodel = Sparam.dmodel;
if (dmodel == 's')
    [RL] = LMmodel(Sparam,nLloyd);
elseif (dmodel == 'r')
    [RL, Sparam.df] = SigProc(Sparam);
end

% Creates ambientnoise and adds it to the synthetic data if wanted
noi = Sparam.noi;
if (( dmodel == 's') && (noi == 'n'))
    f = Sparam.freq;
    SNRN = 5;                       % SNR parameter
    Nadj = ((mean(abs(RL(:,t_num/2 +1)))/(10^(SNRN/20)))/mean(AmbNoise(f))); % Calculated noise adjustment to scale the noise to the chosen SNR
    Sparam.Nadj = Nadj;             % Noise adjustment
    Sparam.Nvar = 0.5;              % Noise variance
end
%% Run search

nk = 10

for k = 1:nk
    k
    
    %Add noise for each iterasjon
    if (( dmodel == 's') && (noi == 'n'))
        [RL] = LMmodelNoise(Sparam,nLloyd); % Add noise to the synthetic data
    end
    
    for flg = 1:2
        idx = flg;
      
        %% Set ASHS-parameters
        if flg == 1
            ASHSparams.HMS=60;
            ASHSparams.HMCR = 0.8;
            ASHSparams.PAR = 0.4;
            ASHSparams.b = (Pparams.ub-Pparams.lb)/100;
            
            
            ASHSparams.track = zeros(9,1);       % Tracking the ASHS search algorithm where the parameter and error is stored 
            ASHSparams.nglobal = 171;
            ASHSparams.nlocal=254;
            ASHSparams.Lincfact=0.3;
            
            % Call Adaptive Simplex Harmony Search
            tic;
            [xbest,fbest, funccalls, trackHS] = ASHS(ASHSparams,Pparams,RL,Sparam);
            toc;
        end
        
        %% Set ASDE-parameters
        if flg == 2
            
%             ASDEparams.popsize= 30; 
%             ASDEparams.cr=1;
%             ASDEparams.F_weight=0.5;

%           the following from Duong
            ASDEparams.popsize = 100; 
            ASDEparams.cr = 0.8;
            ASDEparams.F_weight = 0.6;
            
            ASDEparams.track = zeros(9,1);       % Tracking the ASDE search algorithm where the parameter and error is stored 
            ASDEparams.nglobal=98;               
            ASDEparams.nlocal=380;               
            ASDEparams.Lincfact=0.3;
           
            % Call Adaptive Simplex Differintial Evolution
            tic;
            [xbest, fbest, funccalls, trackDE] = ASdeopt(ASDEparams,Pparams,RL,Sparam);
            toc;
        end
        
        %% Store the error, parameter, function calls and time
        e(k,idx) = fbest;
        x(k,:,idx) = xbest;
        fc(k,idx) = funccalls;
        tim(k,idx) = toc;
        
    end
end

%% plot

clear xp xs xd
nk = length(x(:,1,1))
xp (1:nk,1:2)=x(:,1,:);
xs (1:nk,1:2)=x(:,2,:);
xd (1:nk,1:2)=x(:,3,:);

% Plots the error, number of function calls and time for each iterasjon
figure
%title('Cost function Error, functioncalls and time')
k = 1:nk;
for idx = 1:2 
    subplot(3,2,idx)
    plot(k, e(:,idx),'DisplayName','Cost function error'), hold
    plot(k,mean(e(:,idx))*ones(1,nk),'r','DisplayName','mean')
    xlabel('Iterasjon')
    ylabel('error')
    
    subplot(3,2,2+idx)
    plot(k,fc(:,idx),'DisplayName','Functioncalls'), hold
    plot(k,mean(fc(:,idx))*ones(1,nk),'r','DisplayName','mean')
    xlabel('Iterasjon')
    ylabel('functioncalls')
    %ylim([0 50000+1000])
    
    subplot(3,2,4+idx)
    plot(k,tim(:,idx),'DisplayName','Time'), hold
    plot(k, mean(tim(:,idx))*ones(1,nk),'r','DisplayName','mean')
    ylim([0 max(max((tim))+10)])
    xlabel('Iterasjon')
    ylabel('Time (s)')
end
em = mean(e)
fcm = mean(fc)
timm=mean(tim)

% Plots the estimated parameters for each iterasjon
figure
%title('Parameter value, left ASHS and right ASDE')
for idx = 1:2
    subplot(3,2,idx)
    plot(k, xp(:,idx),'DisplayName','Estimated'), hold
    plot(k,mean(xp(:,idx))*ones(1,nk),'r','DisplayName','mean')
    plot(k,Sparam.vs*ones(1,nk),'g','DisplayName','real')
    ylim([Pparams.lb(1) Pparams.ub(1)])
    xlabel('Iterasjon')
    ylabel('Boat Speed (m/s)')
    
    subplot(3,2,2+idx)
    plot(k,xs(:,idx),'DisplayName','Estimated'), hold
    plot(k,mean(xs(:,idx))*ones(1,nk),'r','DisplayName','mean')
    plot(k,Sparam.dcpa*ones(1,nk),'g','DisplayName','real')
    ylim([Pparams.lb(2) Pparams.ub(2)])
    xlabel('Iterasjon')
    ylabel('Range (m)')
    
    subplot(3,2,4+idx)
    plot(k,xd(:,idx),'DisplayName','Estimated'), hold
    plot(k,mean(xd(:,idx))*ones(1,nk),'r','DisplayName','mean')
    plot(k,Sparam.d(2)*ones(1,nk),'g','DisplayName','real')
    ylim([Pparams.lb(3) Pparams.ub(3)])
    xlabel('Iterasjon')
    ylabel('Source Depth(m)')  
end

% Plots the parameter error for each iterasjon
figure
%title('Parameter error, left ASHS and right ASDE')
for idx = 1:2
    subplot(3,2,idx)
    plot(k, abs(xp(:,idx)-Sparam.vs),'DisplayName','Param. Error')
    ylim([0, max(abs(xp(:,idx) - Sparam.vs) + abs(xp(:,idx) - Sparam.vs)*0.2)])
    xlabel('Iterasjon')
    ylabel('Boat Speed (m/s)')
    
    subplot(3,2,2+idx)
    plot(k,abs(xs(:,idx) - Sparam.dcpa),'DisplayName','Param. Error')
    ylim([0, max(abs(xs(:,idx) - Sparam.dcpa) + abs(xs(:,idx) - Sparam.dcpa)*0.2)])
    xlabel('Iterasjon')
    ylabel('Range (m)')
    
    subplot(3,2,4+idx)
    plot(k,abs(xd(:,idx) - Sparam.d(2)),'DisplayName','Param. Error')
    ylim([0, max(abs(xd(:,idx) - Sparam.d(2)) + abs(xd(:,idx) - Sparam.d(2))*0.2)])
    xlabel('Iterasjon')
    ylabel('Source Depth (m)')
end

xpm = mean(xp)
xsm = mean(xs)
xdm = mean(xd)


% Plot tracking of paramters (sensetivity) for all 8 parameters
if (length(Pparams.ub) == 8)
    figure
    subplot(4,2,1)
    scatter(trackHS(2,:),trackHS(1,:),'.');
    xlim([Pparams.lb(1) Pparams.ub(1)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('v_{s} (m/s)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,2)
    scatter(trackHS(3,:),trackHS(1,:),'.')
    xlim([Pparams.lb(2) Pparams.ub(2)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('d_{cpa} (m)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,3)
    scatter(trackHS(4,:),trackHS(1,:),'.')
    xlim([Pparams.lb(3) Pparams.ub(3)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('d_{s} (m)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,4)
    scatter(trackHS(5,:),trackHS(1,:),'.')
    xlim([Pparams.lb(4) Pparams.ub(4)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('d_{b}/d_{w} (m)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,5)
    scatter(trackHS(6,:),trackHS(1,:),'.')
    xlim([Pparams.lb(5) Pparams.ub(5)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('c_{w} (m/s)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,6)
    scatter(trackHS(7,:),trackHS(1,:),'.')
    xlim([Pparams.lb(6) Pparams.ub(6)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('c_{s} (m/s)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,7)
    scatter(trackHS(8,:),trackHS(1,:),'.')
    xlim([Pparams.lb(7) Pparams.ub(7)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('\rho_{w} (kg/{m}^{3}')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,8)
    scatter(trackHS(9,:),trackHS(1,:),'.')
    xlim([Pparams.lb(8) Pparams.ub(8)])
    %title('HS Sensitivity')
    ylim([min(trackHS(1,:)) max(trackHS(1,:))])
    xlabel('\rho_{s} (kg/{m}^{3}')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)

    figure
    subplot(4,2,1)
    scatter(trackDE(2,:),trackDE(1,:),'.')
    xlim([Pparams.lb(1) Pparams.ub(1)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('v_{s} (m/s)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,2)
    scatter(trackDE(3,:),trackDE(1,:),'.')
    xlim([Pparams.lb(2) Pparams.ub(2)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('d_{cpa} (m)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,3)
    scatter(trackDE(4,:),trackDE(1,:),'.')
    xlim([Pparams.lb(3) Pparams.ub(3)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('d_{s} (m)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,4)
    scatter(trackDE(5,:),trackDE(1,:),'.')
    xlim([Pparams.lb(4) Pparams.ub(4)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('d_{b}/d_{w} (m)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,5)
    scatter(trackDE(6,:),trackDE(1,:),'.')
    xlim([Pparams.lb(5) Pparams.ub(5)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('c_{w} (m/s)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,6)
    scatter(trackDE(7,:),trackDE(1,:),'.')
    xlim([Pparams.lb(6) Pparams.ub(6)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('c_{s} (m/s)')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,7)
    scatter(trackDE(8,:),trackDE(1,:),'.')
    xlim([Pparams.lb(7) Pparams.ub(7)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('\rho_{w} (kg/{m}^{3}')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
    subplot(4,2,8)
    scatter(trackDE(9,:),trackDE(1,:),'.')
    xlim([Pparams.lb(8) Pparams.ub(8)])
    %title('DE Sensitivity')
    ylim([min(trackDE(1,:)) max(trackDE(1,:))])
    xlabel('\rho_{s} (kg/{m}^{3}')
    ylabel('Error (log)')
    h = get(gca,'children');
    set(h, 'sizedata', 20)
    
end
